<?php

namespace Drupal\cision_block\Plugin\Block;

use Drupal\cision_block\Item\CisionItem;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\UrlHelper;
use GuzzleHttp\Client;

/**
 * Provides a 'Cision' block.
 *
 * @Block(
 *   id = "cision_block",
 *   admin_label = @Translation("Cision block"),
 * )
 */
class CisionBlock extends BlockBase {
  /**
   * The default cache lifetime for the block.
   */
  const DEFAULT_CACHE_LIFETIME = 60 * 5;

  /**
   * The default number of items in the feed.
   */
  const DEFAULT_ITEM_COUNT = 50;

  /**
   * The default number of items per page.
   */
  const DEFAULT_ITEMS_PER_PAGE = 0;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'source' => '',
      'count' => CisionBlock::DEFAULT_ITEM_COUNT,
      'items_per_page' => CisionBlock::DEFAULT_ITEMS_PER_PAGE,
      'display_images' => TRUE,
      'cachetime' => CisionBlock::DEFAULT_CACHE_LIFETIME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    $form['source'] = [
      '#title' => $this->t('Cision Feed source'),
      '#type' => 'url',
      '#required' => TRUE,
      '#default_value' => $this->configuration['source'],
    ];
    $form['count'] = [
      '#title' => $this->t('Number of feed items'),
      '#type' => 'number',
      '#required' => TRUE,
      '#default_value' => $this->configuration['count'],
    ];
    $form['items_per_page'] = [
      '#title' => $this->t('Items per page'),
      '#description' => $this->t('The number of items to display per page. Enter 0 to disable pagination.'),
      '#type' => 'number',
      '#required' => FALSE,
      '#default_value' => $this->configuration['items_per_page'],
    ];
    $form['display_images'] = [
      '#title' => $this->t('Display images'),
      '#description' => $this->t('Show images for each feed item.'),
      '#type' => 'checkbox',
      '#required' => FALSE,
      '#default_value' => $this->configuration['display_images'],
    ];
    $form['cachetime'] = [
      '#title' => $this->t('Cache lifetime'),
      '#description' => $this->t('The cache lifetime in seconds.'),
      '#type' => 'number',
      '#required' => FALSE,
      '#default_value' => $this->configuration['cachetime'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['source'] = $form_state->getValue('source');
    $this->configuration['count'] = $form_state->getValue('count');
    $this->configuration['items_per_page'] = $form_state->getValue('items_per_page');
    $this->configuration['display_images'] = $form_state->getValue('display_images');
    $this->configuration['cachetime'] = $form_state->getValue('cachetime');
  }

  /**
   * Returns markup for a pager if enabled.
   *
   * @param array $items
   *   An array of feed items.
   * @param int $items_per_page
   *   The maximum number of feed items on each page.
   *
   * @return string
   *   Markup for the pager.
   */
  protected function getPager(array &$items, $items_per_page): string {
    $output = '';
    if ($items_per_page > 0) {
      $max = ceil(count($items) / $items_per_page);
      $params = UrlHelper::filterQueryParameters(\Drupal::request()->query->all());
      $active = (isset($params['id']) && $params['id'] == 'cision_block' ? $params['page'] : 0);
      if ($max > 1) {
        $output = '<ul class="cision-pager">';
        for ($i = 0; $i < $max; $i++) {
          $output .= '<li><a href="?id=cision_block&page=' . $i . '"' . ($active == $i ? ' class="active"' : '') . '>' . ($i + 1) . '</a></li>';
        }

        if ($active >= 0 && $active < $max) {
          $items = array_slice($items, $active * $items_per_page, $items_per_page);
        }
        $output .= '</ul>';
      }
    }
    return $output;
  }

  /**
   * {@inheritdoc}
   *
   */
  public function build(): array {
    try {
      $items = $this->getFeeds();
    } catch (\Exception $exception) {
      return [];
    }
    $pager = '';
    // TODO: No need to generate pager if items_per_page is 0
    if (count($items)) {
      $pager = $this->getPager($items, $this->configuration['items_per_page']);
    }
    // TODO: Could be moved to the actual mapping
    if (!$this->configuration['display_images']) {
      $items = array_map(function ($item) {
        $item->Images = [];
        return $item;
      }, $items);
    }
    return [
      '#theme' => 'cision_block',
      '#items' => $items,
      '#pager' => $pager,
      '#attached' => [
        'library' => [
          'cision_block/cision_block-module',
        ],
      ],
      '#cache' => [
        'max-age' => 1,// $this->configuration['cachetime'],
        'contexts' => [
          'url.path',
          'url.query_args',
        ],
      ],
    ];
  }

  /**
   * Mapping function.
   *
   * @todo Collect and sanitize basic fields, add hook so that a user can add more field data.
   *
   * @param \stdClass $feed
   *   Object containing all feed items.
   *
   * @return array
   *   An array of mapped items.
   */
  protected function sourceMapping(\stdClass $feed): array {
    $items = NULL;
    if (isset($feed->Releases)) {
      foreach ($feed->Releases as $release) {
        $item = new CisionItem();
        $item->set('Id', $release->Id);
        $item->set('Title', $release->Title);
        $item->set('HtmlTitle', $release->HtmlTitle);
        $item->set('Header', $release->Header);
        $item->set('HtmlHeader', $release->HtmlHeader);
        $item->set('PublishDate', strtotime($release->PublishDate));
        $item->set('LastChangeDate', strtotime($release->LastChangeDate));
        $item->set('Intro', $release->Intro);
        $item->set('Summary', text_summary($release->Intro ? $release->Intro : $release->Body));
        $item->set('HtmlIntro', $release->HtmlIntro);
        $item->set('Body', $release->Body);
        $item->set('HtmlBody', $release->HtmlBody);
        $item->set('CountryCode', $release->CountryCode);
        $item->set('LanguageCode', $release->LanguageCode);
        $item->set('IsRegulatory', $release->IsRegulatory);
        $item->set('InformationType', $release->InformationType);
        $item->set('SocialMediaPitch', $release->SocialMediaPitch ?? '');
        $item->set('Contact', $release->Contact);
        $item->set('HtmlContact', $release->HtmlContact);
        $item->set('Images', $release->Images);
        $item->set('Files', $release->Files);
        $item->set('CisionWireUrl', $release->CisionWireUrl);
        $item->set('Keywords', $release->Keywords);
        $item->set('Categories', $release->Categories);
        $item->set('CompanyInformation', $release->CompanyInformation);
        $item->set('HtmlCompanyInformation', $release->HtmlCompanyInformation);
        $item->set('QuickFacts', $release->QuickFacts);
        $item->set('LogoUrl', $release->LogoUrl);
        $item->set('Quotes', $release->Quotes);
        $item->set('ExternalLinks', $release->ExternalLinks);
        $item->set('EmbeddedItems', $release->EmbeddedItems);
        $item->set('Videos', $release->Videos);
        $items[] = $item;
      }
    }
    return $items;
  }

  /**
   * Returns a list of feeds from cision.
   *
   * @return array
   *   All feed items.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  protected function getFeeds(): array {
    $query_args = http_build_query([
      'detailLevel' => 'detail',
      'pageSize' => $this->configuration['count'],
      'format' => 'json',
    ]);
    $client = new Client();
    $request = $client->request('GET', $this->configuration['source'] . '?' . $query_args);
    $response = (string) $request->getBody();
    $feed = json_decode($response);
    return $this->sourceMapping($feed);
  }

}
